// vim: ts=4 sw=4 cindent foldmethod=marker
//
// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005-2006
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//

function dump_rdf (data_source) {/*{{{*/
	var ser=Components.classes["@mozilla.org/rdf/xml-serializer;1"].createInstance(Components.interfaces.nsIRDFXMLSerializer);
	ser.init(data_source);
	var stream = {
		xml : '',
		close : function(){},
		flush : function(){},
		write : function (buffer,count) { 
			this.xml = this.xml+buffer; 
			return count; 
		},
		writeFrom : function (stream,count){},
		isNonBlocking: false
	};
	ser.QueryInterface(Components.interfaces.nsIRDFXMLSource).Serialize(stream);
	XXX("stream.xml",stream.xml);
}/*}}}*/
function flock_typomaticNormalizeQuery (input) {/*{{{*/
    var output;
    var in_quotes = false;

    // strip leading spaces
    while (input.length && input[0]==' ') {
        // chop off the first character
        input = input.substr (1);
    }

	// Convert to lowercase for the wildcard search
    input = input.toLowerCase();
	
    // add leading plus
    output = '+';
    while (input.length) {
        if (input[0] == '"') {
            if (in_quotes) {
                in_quotes = false;
            } else {
                in_quotes = true;
                // FIXME: remove spaces from the start of quoted strings
            }
        }

        // escape special chars
        if ("&|!(){}[]^~*?:\\".indexOf (input[0]) >= 0) {
            output = output + '\\';
        }

        output = output + input[0];

        if (!in_quotes && input[0]==' ' && input.length>1) {
            // put a plus before every word
            output = output + '+';
        }

        input = input.substr (1);

    }

    // remove trailing spaces
    var removed_spaces = false;
    while (output.length>0 && output[output.length-1]==' ') {
        output = output.substr (0, output.length-1);
        removed_spaces = true;
    }

    // if we're in a quoted string we should close that
    if (in_quotes) {
        output = output + '"';
    }
	
    // if theres an incomplete word
    if ((!removed_spaces) && (output[output.length-1] != '"')) {
        // search for partial matches
		output = output + '*';
    }

    return output;
}/*}}}*/


// new-style stuff
flock.search = {/*{{{*/
    // a life spent seeking a perfect blossom
    // would not be a wasted life
}/*}}}*/

flock.search.__defineGetter__("service", function () {/*{{{*/
    return flock.xpcom.getService(
        '@flock.com/lucene/indexer;1',
        'flockILuceneIndexer'
    );
});/*}}}*/


// XXX andy: this should inherit from a Deferred class
flock.search.SearchCallback = function (aProps) {/*{{{*/
    flock.Base.call(this, aProps);
}/*}}}*/
dojo.inherits(flock.search.SearchCallback, flock.Base);
dojo.lang.extend(flock.search.SearchCallback, {/*{{{*/
    FoundResults: function (aCount, aResults) {
        this.onResult(aResults);
    },
    onResult: function (aData) {
    },
    onError: function (aError) {
        // XXX andy: nothing seems to return an error yet
    }
});/*}}}*/

flock.search.normalizeQuery = function (aQuery) {/*{{{*/
    var output;
    var in_quotes = false;

    // strip leading spaces
    while (aQuery.length && aQuery[0]==' ') {
        // chop off the first character
        aQuery = aQuery.substr (1);
    }
    
    // Convert to lowercase for the wildcard search
    aQuery = aQuery.toLowerCase();


    // add leading plus
    output = '+';
    while (aQuery.length) {
        if (aQuery[0] == '"') {
            if (in_quotes) {
                in_quotes = false;
            } else {
                in_quotes = true;
                // FIXME: remove spaces from the start of quoted strings
            }
        }

        // escape special chars
        if ("&|!(){}[]^~*?:\\".indexOf (aQuery[0]) >= 0) {
            output = output + '\\';
        }

        output = output + aQuery[0];

        if (!in_quotes && aQuery[0]==' ' && aQuery.length>1) {
            // put a plus before every word
            output = output + '+';
        }

        aQuery = aQuery.substr (1);

    }

    // remove trailing spaces
    var removed_spaces = false;
    while (output.length>0 && output[output.length-1]==' ') {
        output = output.substr (0, output.length-1);
        removed_spaces = true;
    }

    // if theres an incomplete word
    if (!removed_spaces) {
        // search for partial matches
        output = output + '*';
    }

    // if we're in a quoted string we should close that
    if (in_quotes) {
        output = output + '"';
    }
    
    return output;
}/*}}}*/

flock.search.search = function (aQuery, aMaxResults, aCallback) {/*{{{*/
    var query = flock.search.normalizeQuery(aQuery);
    var maxResults = aMaxResults || 100;
    var callback = aCallback || new flock.search.SearchCallback();
    var dataSource = flock.search.service.Search(query, maxResults, callback);
    return dataSource;
}/*}}}*/
