// vim: ts=4 sw=4 cindent expandtab
//
// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005-2006
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//
var panePhotos = {};

panePhotos.unload = function() {
    panePhotos.mApiMgr.removeListener(panePhotos.mListener);
}

panePhotos.load = function() {
    this.mApiMgr = Components.classes['@flock.com/photo-api-manager;1?']
        .getService(Components.interfaces.flockIPhotoAPIManager);
    this.setupList();
    this.updateCommands();
    this.mListener = {
        onAPIStateChange: function(aAPI) {
            panePhotos.updateCommands();
        },
        onError: function(aError) {
            alert(aError.errorString);
        },
    }
    this.mApiMgr.addListener(this.mListener);
    window.addEventListener('unload', panePhotos.unload, false);
}

panePhotos.getServices = function(aAll) {
    var rval = {};
    if(aAll) {
        var services = this.mApiMgr.services;
    }
    else {
        var services = this.mApiMgr.preferredServices;
    }
    while(services.hasMoreElements()) {
        var svc = services.getNext();
        rval[svc.shortName] = svc;
    }
    return rval;
}

panePhotos.setupList = function() {
    var preferredServices = this.getServices();
    var listing = $("listing");
    
    while(listing.childNodes.length) {
        try {
            listing.lastChild.removeListener();
        } catch(e) { }
        listing.removeChild(listing.lastChild);
    }

    for(var p in preferredServices) {
        var svc = preferredServices[p];
        var serviceName = svc.serviceName; if(!serviceName) continue;//tmp workaround
        var account = document.createElement("photoAccount");
        account.service = svc;
        listing.appendChild(account);
    }
    
    if (!listing.childNodes.length) {
        var noAccountMsg = document.createElement('label');
        noAccountMsg.setAttribute('value', "No photo service accounts setup.");
        listing.appendChild(noAccountMsg);
    }
}

panePhotos.addPhotoAccount = function() {
}

panePhotos.doCommand = function(aID) {
    var cmd = panePhotos[aID];
    cmd.exec();
    panePhotos.updateCommands();
}

panePhotos.updateCommands = function() {
    var set = $("photoPrefCommands");
    for(var i=0;i<set.childNodes.length;++i) {
        var element = set.childNodes.item(i);
        var cmd = panePhotos[element.getAttribute("id")];
        if(cmd.isEnabled()) {
            element.removeAttribute("disabled");
        }
        else {
            element.setAttribute("disabled", true);
        }
    }
}

panePhotos.cmd_addPhotoAccount = {
    isEnabled: function() {
        var services = panePhotos.getServices(true);
        var preferredServices = panePhotos.getServices();
        var a=0;
        var b=0;
        for(var s in services) ++a;
        for(var s in preferredServices) ++b;
        if(a==b) return false;
        return true;
    },
    exec: function() {
        window.openDialog("chrome://browser/content/flock/photo/photoAccountWizard.xul", "PhotoAccountWizard", "modal, centerscreen");
        panePhotos.setupList();
    },
}
